%./ "Latent Common Return Volatility Factors: Capturing Elusive Predictive
%./ Accuracy Gains When Forecasting Volatility"

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%     Main Programme    %%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
load Dataset_Pool_10min.mat
%./ Parametrization and Data Loading ./%
Rolling_Win = 630;              %./ Rolling-Window Esitmation: 630 Days
Freq = 1/39;                   %./ Sampling Frequency: 10-min 
Y = Dataset_Pool_10min(:, 216);  %./ Loading the target asset to be predicted (SPY)
Pool = [Dataset_Pool_10min(:, 1:215),...  %./ Loading other individual stocks (Num: 274)
    Dataset_Pool_10min(:, 217:263), Dataset_Pool_10min(:,273:277)];
[rows, cols] = size(Pool);

%./ Estimate the time series of (annualized) daily IV's for all assets ./%
IV = zeros(rows*Freq, 1);           %./ For the target asset
IV_Pool = zeros(rows*Freq, cols);   %./ For the asset pool
for j = 1:rows*Freq
    temp = Y((j-1)/Freq+1:j/Freq);
    temp_pool = Pool((j-1)/Freq+1:j/Freq,:);
    temp_iv = (252)*Integ_Vol(temp, Freq, 1);
    temp_iv_pool = (252)*Integ_Vol(temp_pool, Freq, 1);
    IV(j) = temp_iv;
    IV_Pool(j,:) = temp_iv_pool;   
end

%./ Forecasting Experiments ./%
Pred_Rolling_1day = zeros(380,1);
InSampleR2_roll_1day = zeros(380,1); 
OutSampleADR_roll_1day = zeros(380,1);
Pred_PC_Lasso_Rolling_1day = zeros(380,1);
InSampleR2_PC_Lasso_roll_1day = zeros(380,1); 
OutSampleADR_PC_Lasso_roll_1day = zeros(380,1);
Pred_SPC_Lasso_Rolling_1day = zeros(380,1);
InSampleR2_SPC_Lasso_roll_1day = zeros(380,1); 
OutSampleADR_SPC_Lasso_roll_1day = zeros(380,1);
Pred_PC_EN1_Rolling_1day = zeros(380,1);
InSampleR2_PC_EN1_roll_1day = zeros(380,1); 
OutSampleADR_PC_EN1_roll_1day = zeros(380,1);
Pred_SPC_EN1_Rolling_1day = zeros(380,1);
InSampleR2_SPC_EN1_roll_1day = zeros(380,1); 
OutSampleADR_SPC_EN1_roll_1day = zeros(380,1);
Pred_PC_EN2_Rolling_1day = zeros(380,1);
InSampleR2_PC_EN2_roll_1day = zeros(380,1); 
OutSampleADR_PC_EN2_roll_1day = zeros(380,1);
Pred_SPC_EN2_Rolling_1day = zeros(380,1);
InSampleR2_SPC_EN2_roll_1day = zeros(380,1); 
OutSampleADR_SPC_EN2_roll_1day = zeros(380,1);

NumFactors_Lasso_PC = zeros(380,1);
NumFactors_Lasso_SPC = zeros(380,1);
NumFactors_EN1_PC = zeros(380,1);
NumFactors_EN1_SPC = zeros(380,1);
NumFactors_EN2_PC = zeros(380,1);
NumFactors_EN2_SPC = zeros(380,1);

Files_Pool = [Files(1:221,:); Files(223:270,:); Files(280:284,:)];

Ind_Disp_Lasso = {};
Ind_Disp_EN1 = {};
Ind_Disp_EN2 = {};

tic
for d = 250:629  %./ d = 250 => July 1, 2009;  d = 629 => December 31, 2010
    
%./ The Benchmark Model %./    
    target = IV(d+Rolling_Win);
    temp_iv_roll = IV(d:d+Rolling_Win-1);       
    temp_iv_roll_pool = IV_Pool(d:d+Rolling_Win-1,:);    
    X1 = temp_iv_roll(22:Rolling_Win-1);
    X2 = []; X3 = [];
    X4 = temp_iv_roll_pool(22:Rolling_Win-1,:);

    for k = 1: length(X1)
        temp_week = mean(temp_iv_roll(k+17: k+21));
        temp_month = mean(temp_iv_roll(k: k+21));                           
        X2 = [X2; temp_week];
        X3 = [X3; temp_month];                            
    end

    tempX = [ones(length(X1),1), X1, X2, X3];
    tempY = temp_iv_roll(22+1:length(temp_iv_roll));
    beta_est = (tempX'*tempX)\(tempX'*tempY);
    Pred = [1, temp_iv_roll(length(temp_iv_roll)), mean(temp_iv_roll(length(temp_iv_roll)-4 :length(temp_iv_roll))), ...
                   mean(temp_iv_roll(length(temp_iv_roll)-21 :length(temp_iv_roll)))]*beta_est;

    InSampleR2_roll_1day(d-249) = 1 - sum((tempY-tempX*beta_est).^2)/sum((tempY-mean(tempY)).^2);
    Pred_Rolling_1day(d-249) = Pred;
    OutSampleADR_roll_1day(d-249) = abs(((Pred)-target)/target);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%./ Proposed Models

%./ Step 1: Variable Selection
    Select_pool = [tempX , X4];
    
%./ LASSO
    [Coeff_Lasso, Info_Lasso] = lasso(Select_pool, tempY, 'CV', 10);        
    if length(find(Coeff_Lasso(5:size(Coeff_Lasso,1),Info_Lasso.IndexMinMSE)~=0))>=round(size(Pool,2)*0.15)
       Coeff_Lasso = Coeff_Lasso(5:size(Coeff_Lasso,1), Info_Lasso.IndexMinMSE);  
    else
       sort_mse = sort(Info_Lasso.MSE); 
       z = 2;
       while length(find(Coeff_Lasso(5:size(Coeff_Lasso,1), find(Info_Lasso.MSE==sort_mse(z)))~=0))<round(size(Pool,2)*0.15)
             z = z + 1;
       end
       Coeff_Lasso = Coeff_Lasso(5:size(Coeff_Lasso,1), find(Info_Lasso.MSE==sort_mse(z)));  
    end

%./ Elastic net with alpha = 0.2 (EN-1)
    [Coeff_EN1, Info_EN1] = lasso(Select_pool, tempY, 'Alpha', 0.2, 'CV', 10); 
    if length(find(Coeff_EN1(5:size(Coeff_EN1,1),Info_EN1.IndexMinMSE)~=0))>=round(size(Pool,2)*0.15)
       Coeff_EN1 = Coeff_EN1(5:size(Coeff_EN1,1), Info_EN1.IndexMinMSE);  
    else
       sort_mse = sort(Info_EN1.MSE); 
       z = 2;
       while length(find(Coeff_EN1(5:size(Coeff_EN1,1), find(Info_EN1.MSE==sort_mse(z)))~=0))<round(size(Pool,2)*0.15)
            z = z + 1;
       end
       Coeff_EN1 = Coeff_EN1(5:size(Coeff_EN1,1), find(Info_EN1.MSE==sort_mse(z)));  
    end
    
%./ Elastic net with alpha = 0.6 (EN-2)
    [Coeff_EN2, Info_EN2] = lasso(Select_pool, tempY, 'Alpha', 0.6, 'CV', 10); 
    if length(find(Coeff_EN2(5:size(Coeff_EN2,1),Info_EN2.IndexMinMSE)~=0))>=round(size(Pool,2)*0.15)
       Coeff_EN2 = Coeff_EN2(5:size(Coeff_EN2,1), Info_EN2.IndexMinMSE);  
    else
       sort_mse = sort(Info_EN2.MSE); 
       z = 2;
       while length(find(Coeff_EN2(5:size(Coeff_EN2,1), find(Info_EN2.MSE==sort_mse(z)))~=0))<round(size(Pool,2)*0.15)
            z = z + 1;
       end
       Coeff_EN2 = Coeff_EN2(5:size(Coeff_EN2,1), find(Info_EN2.MSE==sort_mse(z)));  
    end
    
%./ Obtain the subset of assets after variable selection     
    
    DailyRVInd_Lasso = find(Coeff_Lasso~=0); 
    DailyRVInd_EN1 = find(Coeff_EN1~=0);
    DailyRVInd_EN2 = find(Coeff_EN2~=0);
    
    Std_SelectIV_Lasso = sqrt(mean(temp_iv_roll_pool(:, DailyRVInd_Lasso)));
    Std_SelectIV_EN1 = sqrt(mean(temp_iv_roll_pool(:, DailyRVInd_EN1)));
    Std_SelectIV_EN2 = sqrt(mean(temp_iv_roll_pool(:, DailyRVInd_EN2)));
    
    for p1 = 1:length(DailyRVInd_Lasso)
       temp1 = Files_Pool(DailyRVInd_Lasso(p1)).name; 
       SelectStocks_Lasso{p1, d-249} = temp1(6:end-23);
    end
    
    for p2 = 1:length(DailyRVInd_EN1)
       temp2 = Files_Pool(DailyRVInd_EN1(p2)).name;
       SelectStocks_EN1{p2, d-249} = temp2(6:end-23); 
    end
    
    for p3 = 1:length(DailyRVInd_EN2)
       temp3 = Files_Pool(DailyRVInd_EN2(p3)).name;
       SelectStocks_EN2{p3, d-249} = temp3(6:end-23); 
    end    
    
    Pool_Lasso = Pool((d-1)/Freq+1: (d+Rolling_Win-1)/Freq, DailyRVInd_Lasso);
    Pool_EN1 = Pool((d-1)/Freq+1: (d+Rolling_Win-1)/Freq, DailyRVInd_EN1);
    Pool_EN2 = Pool((d-1)/Freq+1: (d+Rolling_Win-1)/Freq, DailyRVInd_EN2);
    
    Pool_Lasso = Pool_Lasso(21/Freq+1:size(Pool_Lasso,1), :);
    Pool_EN1 = Pool_EN1(21/Freq+1:size(Pool_EN1,1), :);
    Pool_EN2 = Pool_EN2(21/Freq+1:size(Pool_EN2,1), :);
    
%./ Construct Factors and Calculate the Daily IV of Factors  

    RVpool_PC_Lasso = zeros(length(X1)+1, 20);
    RVpool_SPC_Lasso = zeros(length(X1)+1, 20);                     
    RVpool_PC_EN1 = zeros(length(X1)+1, 20);
    RVpool_SPC_EN1 = zeros(length(X1)+1, 20);  
    RVpool_PC_EN2 = zeros(length(X1)+1, 20);
    RVpool_SPC_EN2 = zeros(length(X1)+1, 20);
    
    for j = 1:length(X1)+1
        
        temp_lasso = Pool_Lasso((j-1)/Freq+1: j/Freq,:);
        temp_en1 = Pool_EN1((j-1)/Freq+1: j/Freq,:);
        temp_en2 = Pool_EN2((j-1)/Freq+1: j/Freq,:);
        
        thresh_lasso = threshold(temp_lasso, Freq);
        thresh_en1 = threshold(temp_en1, Freq);
        thresh_en2 = threshold(temp_en2, Freq);
        Ind_lasso = find(sum(thresh_lasso)==0);
        Ind_en1 = find(sum(thresh_en1)==0);
        Ind_en2 = find(sum(thresh_en2)==0);
        thresh_lasso(:, Ind_lasso) = [];
        thresh_en1(:, Ind_en1) = [];
        thresh_en2(:, Ind_en2) = [];
        
        if any(Ind_lasso)~=0
            Ind_Disp_Lasso{d-249, j} = Ind_lasso;
        end
        if any(Ind_en1)~=0
            Ind_Disp_EN1{d-249, j} = Ind_en1;
        end
        if any(Ind_en2)~=0
            Ind_Disp_EN2{d-249, j} = Ind_en2;
        end
        
        Std_Lasso = Std_SelectIV_Lasso; Std_Lasso(:, Ind_lasso) = [];
        Std_EN1 = Std_SelectIV_EN1; Std_EN1(:, Ind_en1) = [];
        Std_EN2 = Std_SelectIV_EN2; Std_EN2(:, Ind_en2) = [];
        
        [PC_Lasso, SPC_Lasso, EigV_Lasso_PC, EigV_Lasso_SPC] = PC_SPC(thresh_lasso, Std_Lasso);
        [PC_EN1, SPC_EN1, EigV_EN1_PC, EigV_EN1_SPC] = PC_SPC(thresh_en1, Std_EN1);
        [PC_EN2, SPC_EN2, EigV_EN2_PC, EigV_EN2_SPC] = PC_SPC(thresh_en2, Std_EN2);
              
        DailyRV_PC_Lasso = (252)*Integ_Vol(PC_Lasso, Freq, 0);
        DailyRV_SPC_Lasso = (252)*Integ_Vol(SPC_Lasso, Freq, 0);
        DailyRV_PC_EN1 = (252)*Integ_Vol(PC_EN1, Freq, 0);
        DailyRV_SPC_EN1 = (252)*Integ_Vol(SPC_EN1, Freq, 0);
        DailyRV_PC_EN2 = (252)*Integ_Vol(PC_EN2, Freq, 0);
        DailyRV_SPC_EN2 = (252)*Integ_Vol(SPC_EN2, Freq, 0);
              
        RVpool_PC_Lasso(j, 1:length(DailyRV_PC_Lasso)) = DailyRV_PC_Lasso;
        RVpool_SPC_Lasso(j, 1:length(DailyRV_SPC_Lasso)) = DailyRV_SPC_Lasso;
        RVpool_PC_EN1(j, 1:length(DailyRV_PC_EN1)) = DailyRV_PC_EN1;
        RVpool_SPC_EN1(j, 1:length(DailyRV_SPC_EN1)) = DailyRV_SPC_EN1;  
        RVpool_PC_EN2(j, 1:length(DailyRV_PC_EN2)) = DailyRV_PC_EN2;
        RVpool_SPC_EN2(j, 1:length(DailyRV_SPC_EN2)) = DailyRV_SPC_EN2;        
      
    end    
    
    KLassoPC = []; 
    KLassoSPC = [];
    KENPC1 = [];
    KENSPC1 = [];      
    KENPC2 = [];
    KENSPC2 = [];            
   
    for k = 1:20
          
         temp_pc_lasso = RVpool_PC_Lasso(:, k);
         temp_spc_lasso = RVpool_SPC_Lasso(:, k);
         temp_pc_en1 = RVpool_PC_EN1(:, k);
         temp_spc_en1 = RVpool_SPC_EN1(:, k);
         temp_pc_en2 = RVpool_PC_EN2(:, k);
         temp_spc_en2 = RVpool_SPC_EN2(:, k);
                       
         if length(find(temp_pc_lasso~=0)) < round(0.9*(length(X1)+1))
           KLassoPC = [KLassoPC; k];
         end
         if length(find(temp_spc_lasso~=0)) < round(0.9*(length(X1)+1))
           KLassoSPC = [KLassoSPC; k];
         end                 
         if length(find(temp_pc_en1~=0)) < round(0.9*(length(X1)+1))
           KENPC1 = [KENPC1; k];
         end
         if length(find(temp_spc_en1~=0)) < round(0.9*(length(X1)+1))
           KENSPC1 = [KENSPC1; k];
         end  
         if length(find(temp_pc_en2~=0)) < round(0.9*(length(X1)+1))
           KENPC2 = [KENPC2; k];
         end
         if length(find(temp_spc_en2~=0)) < round(0.9*(length(X1)+1))
           KENSPC2 = [KENSPC2; k];
         end                
                        
    end

    RVpool_PC_Lasso(:,KLassoPC) = [];
    RVpool_SPC_Lasso(:,KLassoSPC) = [];            
    RVpool_PC_EN1(:,KENPC1) = [];
    RVpool_SPC_EN1(:,KENSPC1) = [];    
    RVpool_PC_EN2(:,KENPC2) = [];
    RVpool_SPC_EN2(:,KENSPC2) = [];    
    
    NumFactors_Lasso_PC(d-249,:) = size(RVpool_PC_Lasso, 2);
    NumFactors_Lasso_SPC(d-249,:) = size(RVpool_SPC_Lasso, 2);
    NumFactors_EN1_PC(d-249,:) = size(RVpool_PC_EN1, 2);
    NumFactors_EN1_SPC(d-249,:) = size(RVpool_SPC_EN1, 2);
    NumFactors_EN2_PC(d-249,:) = size(RVpool_PC_EN2, 2);
    NumFactors_EN2_SPC(d-249,:) = size(RVpool_SPC_EN2, 2);
    
    FactorCoeff_PCA_LASSO{d-249} = EigV_Lasso_PC(:, 1:min(size(EigV_Lasso_PC, 2), size(RVpool_PC_Lasso, 2)));
    FactorCoeff_SPCA_LASSO{d-249} = EigV_Lasso_SPC(:, 1:min(size(EigV_Lasso_SPC, 2), size(RVpool_SPC_Lasso, 2)));
    FactorCoeff_PCA_EN1{d-249} = EigV_EN1_PC(:, 1:min(size(EigV_EN1_PC, 2), size(RVpool_PC_EN1, 2)));
    FactorCoeff_SPCA_EN1{d-249} = EigV_EN1_SPC(:, 1:min(size(EigV_EN1_SPC, 2), size(RVpool_SPC_EN1, 2)));
    FactorCoeff_PCA_EN2{d-249} = EigV_EN2_PC(:, 1:min(size(EigV_EN2_PC, 2), size(RVpool_PC_EN2, 2)));
    FactorCoeff_SPCA_EN2{d-249} = EigV_EN2_SPC(:, 1:min(size(EigV_EN2_SPC, 2), size(RVpool_SPC_EN2, 2)));
    
    tempX_PC_Lasso = [tempX, RVpool_PC_Lasso(1:length(X1),:)];
    tempX_SPC_Lasso = [tempX, RVpool_SPC_Lasso(1:length(X1),:)];
    tempX_PC_EN1 = [tempX, RVpool_PC_EN1(1:length(X1),:)];
    tempX_SPC_EN1 = [tempX, RVpool_SPC_EN1(1:length(X1),:)];    
    tempX_PC_EN2 = [tempX, RVpool_PC_EN2(1:length(X1),:)];
    tempX_SPC_EN2 = [tempX, RVpool_SPC_EN2(1:length(X1),:)];    
    
%./ Finally Estimate Proposed Models and Yield Forecasts

%./ Specification 1 : LASSO + PCA
    beta_est = lsqnonneg(tempX_PC_Lasso, tempY);
    Pred = [1, temp_iv_roll(length(temp_iv_roll)), mean(temp_iv_roll(length(temp_iv_roll)-4 :length(temp_iv_roll))), ...
         mean(temp_iv_roll(length(temp_iv_roll)-21 :length(temp_iv_roll))), RVpool_PC_Lasso(1+length(X1),:)]*beta_est;               
    
    InSampleR2_PC_Lasso_roll_1day(d-249) = 1 - sum((tempY-tempX_PC_Lasso*beta_est).^2)/sum((tempY-mean(tempY)).^2);
    Pred_PC_Lasso_Rolling_1day(d-249) = Pred; 
    OutSampleADR_PC_Lasso_roll_1day(d-249) = abs(((Pred)-target)/target);  

%./ Specification 2 : LASSO + SPCA    
    beta_est = lsqnonneg(tempX_SPC_Lasso, tempY);
    Pred = [1, temp_iv_roll(length(temp_iv_roll)), mean(temp_iv_roll(length(temp_iv_roll)-4 :length(temp_iv_roll))), ...
         mean(temp_iv_roll(length(temp_iv_roll)-21 :length(temp_iv_roll))), RVpool_SPC_Lasso(1+length(X1),:)]*beta_est;               
    
    InSampleR2_SPC_Lasso_roll_1day(d-249) = 1 - sum((tempY-tempX_SPC_Lasso*beta_est).^2)/sum((tempY-mean(tempY)).^2);
    Pred_SPC_Lasso_Rolling_1day(d-249) = Pred; 
    OutSampleADR_SPC_Lasso_roll_1day(d-249) = abs(((Pred)-target)/target);            
        
%./ Specification 3 : EN-1 + PCA    
    beta_est = lsqnonneg(tempX_PC_EN1, tempY);
    Pred = [1, temp_iv_roll(length(temp_iv_roll)), mean(temp_iv_roll(length(temp_iv_roll)-4 :length(temp_iv_roll))), ...
         mean(temp_iv_roll(length(temp_iv_roll)-21 :length(temp_iv_roll))), RVpool_PC_EN1(1+length(X1),:)]*beta_est;               
    
    InSampleR2_PC_EN1_roll_1day(d-249) = 1 - sum((tempY-tempX_PC_EN1*beta_est).^2)/sum((tempY-mean(tempY)).^2);
    Pred_PC_EN1_Rolling_1day(d-249) = Pred; 
    OutSampleADR_PC_EN1_roll_1day(d-249) = abs(((Pred)-target)/target);  
       
%./ Specification 4 : EN-1 + SPCA    
    beta_est = lsqnonneg(tempX_SPC_EN1, tempY);
    Pred = [1, temp_iv_roll(length(temp_iv_roll)), mean(temp_iv_roll(length(temp_iv_roll)-4 :length(temp_iv_roll))), ...
         mean(temp_iv_roll(length(temp_iv_roll)-21 :length(temp_iv_roll))), RVpool_SPC_EN1(1+length(X1),:)]*beta_est;               
    
    InSampleR2_SPC_EN1_roll_1day(d-249) = 1 - sum((tempY-tempX_SPC_EN1*beta_est).^2)/sum((tempY-mean(tempY)).^2);
    Pred_SPC_EN1_Rolling_1day(d-249) = Pred; 
    OutSampleADR_SPC_EN1_roll_1day(d-249) = abs(((Pred)-target)/target);            
            
%./ Specification 5 : EN-2 + PCA 
    beta_est = lsqnonneg(tempX_PC_EN2, tempY);
    Pred = [1, temp_iv_roll(length(temp_iv_roll)), mean(temp_iv_roll(length(temp_iv_roll)-4 :length(temp_iv_roll))), ...
         mean(temp_iv_roll(length(temp_iv_roll)-21 :length(temp_iv_roll))), RVpool_PC_EN2(1+length(X1),:)]*beta_est;               
    
    InSampleR2_PC_EN2_roll_1day(d-249) = 1 - sum((tempY-tempX_PC_EN2*beta_est).^2)/sum((tempY-mean(tempY)).^2);
    Pred_PC_EN2_Rolling_1day(d-249) = Pred; 
    OutSampleADR_PC_EN2_roll_1day(d-249) = abs(((Pred)-target)/target);  

%./ Specification 6 : EN-2 + SPCA    
    beta_est = lsqnonneg(tempX_SPC_EN2, tempY);
    Pred = [1, temp_iv_roll(length(temp_iv_roll)), mean(temp_iv_roll(length(temp_iv_roll)-4 :length(temp_iv_roll))), ...
         mean(temp_iv_roll(length(temp_iv_roll)-21 :length(temp_iv_roll))), RVpool_SPC_EN2(1+length(X1),:)]*beta_est;               
    
    InSampleR2_SPC_EN2_roll_1day(d-249) = 1 - sum((tempY-tempX_SPC_EN2*beta_est).^2)/sum((tempY-mean(tempY)).^2);
    Pred_SPC_EN2_Rolling_1day(d-249) = Pred; 
    OutSampleADR_SPC_EN2_roll_1day(d-249) = abs(((Pred)-target)/target);            
            
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%           
    
    disp(d-249)
    toc    

end

